%**************************************************************************
% Automatic Control Laboratory 
% Brunner Paul
% Date: 31. Januar 2012
% Revision: -
%**************************************************************************
%In this function the continuous time state space model is derived
function [sysc syscr_vma]=...
    continuousTimeSsModelFlexShaft(J)
%Input:
%  J = moment of inertia right and left flywheel
%Output:
%  sysc = continuous time state space models of the plant
%    this model is only used by the simulink model simFlexShaft.mdl
%  sycr = continuous time reduced state space model

figureNr=1; %base figure number
enGraphicOut=0; %enable graphical output: 0=disable, 1=enable

%Motor 1
%-------
R1=6.2; %rotor resistance motor 1 [V/A]
L1=0.75e-3; %motor inductance motor 1 [V*s/A]
J1=J; %inertia motor 1 [kg*m^2/rad]
Kv1=3.125e-5; %viscous friction motor 1 [Nm*s/rad] 
Kt1=4.0e-2; %torque constant [Nm/A]
Ku1=32.5e-3; %back EMF constant motor 1 [V*s/rad] 

%Motor 2
%-------
%R2=R1; %rotor resistance motor 2 [V/A] (not used yet)
%L2=L1; %motor inductance motor 2 [V*s/A] (not used yet)
J2=J1; %inertia motor 2 [kg*m^2/rad] 
Kv2=Kv1; %viscous friction motor 2 [Nm*s/rad] 
Kt2=Kt1; %torque constant motor 2 [Nm/A] 
%Ku2=Ku1; %back EMF constant motor 2 [V*s/rad] (not used yet)

%Spring
%------
f=1.6e-2; %spring constant [Nm/rad] 
d=0.0; %spring damping [Nm*s/rad] 

%Continuous Time State Space Model Plant
%---------------------------------------
%  dx/dt=Ac*x+Bc*u, y=Cc*x+Dc*u
%  x=[x1 ... x5]', u=[u1 u2]', y=[y1 y2 y3 y4 y5]'
%  u1=voltage motor 1, u2=current motor 2
%  y1=x1=velocity motor 1, y2=x2=velocity motor 2
%  y3=x3=position motor 1, y4=x4=position motor 2, y5=x5=current motor 1
sysc.Ac=[-1/J1*(Kv1+d) d/J1 -f/J1 f/J1 Kt1/J1;...
    d/J2 -1/J2*(Kv2+d) f/J2 -f/J2 0;...
    1 0 0 0 0;...
    0 1 0 0 0;...
    -Ku1/L1 0 0 0 -R1/L1]; %system matrix Ac          
sysc.Bc=[0 0; 0 Kt2/J2; 0 0; 0 0; 1/L1 0]; %system matrix Bc
sysc.Cc=[1 0 0 0 0;...
    0 1 0 0 0;...
    0 0 1 0 0;...
    0 0 0 1 0;...
    0 0 0 0 1]; %system matrix Cc
sysc.Dc=[0 0; 0 0; 0 0; 0 0; 0 0]; %system matrix Dc

%Reduced Continuous Time State Space Model Plant
%-----------------------------------------------
%  dx/dt=Ac*x+Bc*w, y=Cc*x+Dc*w
%  x=[x1 ... x3]', w=[u z]'
%  x1=velocity motor 1, x2=velocity motor 1
%  x3=position motor 1 - position motor 2
%  z=current motor 2, u=voltage motor 1, y=x1=velocity motor 1
syscr.Ac=[-1/J1*(Kv1+d+Ku1*Kt1/R1) d/J1 -f/J1;...
    d/J2 -1/J2*(Kv2+d) f/J2;...
    1 -1 0]; %system matrix Ac
syscr.Bc=[Kt1/(J1*R1) 0; 0 Kt2/J2; 0 0]; %system matrix Bc
syscr.Cc=[1 0 0]; %system matrix Cc
syscr.Dc=[0 0]; %system matrix Dc

if enGraphicOut
    sysc_yu=ss(sysc.Ac,sysc.Bc(:,1),sysc.Cc(1,:),sysc.Dc(1,1));
    sysc_yz=ss(sysc.Ac,sysc.Bc(:,2),sysc.Cc(1,:),sysc.Dc(1,2));
    syscr_yu=ss(syscr.Ac,syscr.Bc(:,1),syscr.Cc,syscr.Dc(1,1));
    syscr_yz=ss(syscr.Ac,syscr.Bc(:,2),syscr.Cc,syscr.Dc(1,2));
    w={0.01, 1000}; %frequency [rad/s], w={wmin, wmax}
    %Manipulating Transfer Function
    figure(figureNr); %first figure
    bode(sysc_yu,w);
    hold on;
    bode(syscr_yu,w);
    hold off;
    grid on;
    title('G(s)=Y(s)/U(s)');
    legend('Plant Model','Reduced Plant Model');
    %Disturbance Transfer Fucntion
    figureNr=figureNr+1; 
    figure(figureNr);
    bode(sysc_yz,w);
    hold on;
    bode(syscr_yz,w);
    hold off;
    grid on;
    title('Gz(s)=Y(s)/Z(s)');
    legend('Plant Model','Reduced Plant Model');
    %Pole Zero Map Manipulating Transfer Function Reduced Plant Model
    figureNr=figureNr+1;
    figure(figureNr);
    pzmap(syscr_yu);
    grid on;
    title('Pole Zero Map G(s)=Y(s)/U(s) (Reduced Plant Model)');
    %Pole Zero Map Disturbance Transfer Function Reduced Plant Model
    figureNr=figureNr+1;
    figure(figureNr);
    pzmap(syscr_yu);
    grid on;
    title('Pole Zero Map Gz(s)=Y(s)/Z(s) (Reduced Plant Model)');
end

%Reduced Continuous Time State Space Model With First Order Velocity
%Measurement Approximation
%-------------------------------------------------------------------
%  dx/dt=Ac*x+Bc*w, y=Cc*x+Dc*w
%  x=[x1 ... x4]', w=[u z]'
%  x1=measured velocity motor 1, x2=velocity motor 1
%  x3=velocity motor 2, x4=position motor 1 - position motor 2
%  z=current motor 2, u=voltage motor 1, y=x1=measured velocity motor 1
T1=1/50;
[N,M]=size(syscr.Bc); %N=number of states; M=number of inputs
syscr_vma.Ac=[1/T1.*[-1 syscr.Cc]; zeros(N,1) syscr.Ac]; %system matrix Ac
syscr_vma.Bc=[zeros(1,M); syscr.Bc]; %system matrix Bc
syscr_vma.Cc=[1 zeros(1,N)]; %system matrix Cc
syscr_vma.Dc=zeros(1,M); %system matrix Dc

%Output Structure
%----------------
out.continuousTimeSsModel=sysc;
out.reducedContinuousTimeSsModel=syscr_vma;